/* Copyright (C) 2003 Scott Wheeler <wheeler@kde.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <iostream>
#include <iomanip>
#include <cstdio>
#include <typeinfo>
#include <cstring>
#include <taglib/fileref.h>
#include <taglib/tag.h>
#include <taglib/tpropertymap.h>
#include <taglib/mpegfile.h>//mp3 file
#include <taglib/id3v2tag.h>
#include <taglib/id3v2frame.h>
#include <taglib/attachedpictureframe.h>
//#include <taglib/unsynchronizedlyricsframe.h>
using namespace std;
string get_id3v2_picture(const char * file) {

  TagLib::MPEG::File mp3File(file);
  TagLib::ID3v2::Tag *v2tag = mp3File.ID3v2Tag();
  TagLib::ID3v2::FrameList frame = v2tag->frameListMap()["APIC"];
  TagLib::ID3v2::AttachedPictureFrame * pictureFrame;

  TagLib::ID3v2::FrameList::ConstIterator it= frame.begin();

  for(; it != frame.end() ; it++)
  {
   pictureFrame = static_cast<TagLib::ID3v2::AttachedPictureFrame *> (*it);//cast Frame * to AttachedPictureFrame*
   cout << "toString()     : " << pictureFrame->toString() << endl;
   cout << "textEncoding() : " << pictureFrame->textEncoding() << endl;
   cout << "type()         : " << TagLib::ID3v2::AttachedPictureFrame::Type( pictureFrame->type() ) << endl;
   cout << "mimeType()     :" << pictureFrame->mimeType() << endl;
   cout << "description()  : " << pictureFrame->description() << endl;
   cout << "size()         :" << pictureFrame->size() << endl;
   cout << "data()         :" << pictureFrame->picture().data() << endl;
  }
  return pictureFrame->toString().toCString(true);
}
int main(int argc, char *argv[])
{
  TagLib::String lyrics,label,albumartist;

  for(int i = 1; i < argc; i++) {

    cout << "******************** \"" << argv[i] << "\" ********************" << endl;

    TagLib::FileRef f(argv[i], true, TagLib::AudioProperties::Accurate);

    cout << get_id3v2_picture(argv[i]) << endl;
    if(!f.isNull() && f.tag()) {

      TagLib::Tag *tag = f.tag();
      unsigned int year = tag->year();
      cout << "-- TAG (basic) --" << endl;
      cout << "title   - \"" << tag->title()   << "\"" << endl;
      cout << "artist  - \"" << tag->artist()  << "\"" << endl;
      cout << "album   - \"" << tag->album()   << "\"" << endl;
      cout << "year    - \"" << (unsigned int)tag->year()    << "\"" << endl;
      cout << "comment - \"" << tag->comment() << "\"" << endl;
      cout << "track   - \"" << tag->track()   << "\"" << endl;
      cout << "genre   - \"" << tag->genre().toCString()   << "\"" << endl;

      TagLib::PropertyMap tags = f.file()->properties();

      unsigned int longest = 0;
      for(TagLib::PropertyMap::ConstIterator i = tags.begin(); i != tags.end(); ++i) {
        if (i->first.size() > longest) {
          longest = i->first.size();
        }
      }


      cout << "-- TAG (properties) --" << endl;
      for(TagLib::PropertyMap::ConstIterator i = tags.begin(); i != tags.end(); ++i) {
        for(TagLib::StringList::ConstIterator j = i->second.begin(); j != i->second.end(); ++j) {
          cout << left << std::setw(longest) << i->first << " - " << '"' << *j << '"' << endl;
          if( strcmp(i->first.toCString(true),"ALBUMARTIST") == 0 ) { albumartist = *j; }
         // if( strcmp(i->first.toCString(true), "LYRICS") == 0) { lyrics = *j; }
          if( strcmp(i->first.toCString(true), "TRACKNUMBER") == 0) { cout << "FOUND TRACK NUMBER" << *j << endl;}
          if(i->first == "LABEL") { label = *j; }
        }
      }

    }

    if(!f.isNull() && f.audioProperties()) {


      TagLib::AudioProperties *properties = f.audioProperties();

      int seconds = properties->length() % 60;
      int minutes = (properties->length() - seconds) / 60;

      cout << "-- AUDIO --" << endl;
      cout << "bitrate     - " << properties->bitrate() << endl;
      cout << "sample rate - " << properties->sampleRate() << endl;
      cout << "channels    - " << properties->channels() << endl;
      cout << "length      - " << minutes << ":" << setfill('0') << setw(2) << seconds << endl;

    }
  }
  // .isEmpty()
  //cout << "lyrics:" << lyrics << endl;
  //cout << "label:" << ( label.isEmpty() ? "no Lyrics\n" : label ) << endl;
  return 0;
}
